# -*- coding: utf-8 -*-

from PyQt5.QtCore import QThreadPool, pyqtSlot
from PyQt5.QtWidgets import QLineEdit, QPushButton, QComboBox, QRadioButton
from qgis.gui import QgisInterface

from eagris.controller.worker.add_auth_data import AddAuthDataWorker
from eagris.eagri.auth.ws_key_hash_method import WsKeyHashMethod
from eagris.model.auth import EagriAuthData
from eagris.model.form import EagriModalData
from eagris.controller.panel.panel import QgisEagriPanel
from eagris.eagri.env import EagriEnv
from eagris.controller.constants import EAGRI_MODAL_UI_PATH

try:
    from qgis.PyQt.QtGui import QDialog
except ImportError:
    from qgis.PyQt.QtWidgets import QDialog

from qgis.PyQt import uic
import os


class QgisEagriModal(QDialog):
    nameInput: QLineEdit
    loginInput: QLineEdit
    wsKeyInput: QLineEdit
    szridInput: QLineEdit
    envComboBox: QComboBox
    forceAddBtn: QPushButton
    addAuthBtn: QPushButton
    hashedWsKeyRadioButton: QRadioButton
    plainWsKeyRadioButton: QRadioButton

    def __init__(self, iface: QgisInterface, panel: QgisEagriPanel):
        QDialog.__init__(self)
        ui_path = os.path.join(os.path.dirname(__file__), "..", EAGRI_MODAL_UI_PATH)
        uic.loadUi(ui_path, self)
        self.__iface = iface
        self.__credsTree = panel.credsTree
        self.__addAuthDataWorker = AddAuthDataWorker(self.__iface, self.__credsTree)
        self.__setupTextInputChangeEventListeners()
        self.__fillEnvs()
        self.__threadpool = QThreadPool.globalInstance()

    def __setupTextInputChangeEventListeners(self):
        [t.textChanged.connect(self.__checkAllEagriDataFilledIn) for t in [
            self.loginInput,
            self.wsKeyInput,
            self.szridInput
        ]]

    @pyqtSlot(name="on_addAuthBtn_clicked")
    def __addAuthDataToPanel(self):
        (login, ws_key, szrid, name, env) = self.__collectFormData()
        auth_data = EagriAuthData(
            login=login,
            wskey=ws_key,
            szrid=szrid,
            name=name,
            env=env
        )
        self.__addAuthDataWorker.start(auth_data)

    @pyqtSlot(name="on_forceAddBtn_clicked")
    def __forceAddAuthDataToPanel(self):
        (login, ws_key, szrid, name, env) = self.__collectFormData()
        auth_data = EagriAuthData(
            login=login,
            wskey=ws_key,
            szrid=szrid,
            name=name,
            env=env
        )
        self.__credsTree.addAuthItem(auth_data)

    def __fillEnvs(self):
        self.envComboBox.addItems([EagriEnv.PROD.name, EagriEnv.TEST.name])
        self.envComboBox.setCurrentIndex(0)

    def __collectFormData(self):
        return EagriModalData(
            login=self.loginInput.text(),
            ws_key=self.__collectWsKey(),
            szrid=self.szridInput.text(),
            name=self.nameInput.text(),
            env=EagriEnv.fromStr(self.envComboBox.currentText())
        )

    def __collectWsKey(self):
        hash_method = WsKeyHashMethod.SHA1 if self.hashedWsKeyRadioButton.isChecked() else WsKeyHashMethod.NONE
        return hash_method.hexHash(self.wsKeyInput.text())

    def __checkAllEagriDataFilledIn(self, _text: str):
        self.addAuthBtn.setEnabled(self.__allRequiredItemsAreFilledIn())
        self.forceAddBtn.setEnabled(self.__allRequiredItemsAreFilledIn())

    def __allRequiredItemsAreFilledIn(self):
        name = self.loginInput.text().strip()
        ws_key = self.wsKeyInput.text().strip()
        szrid = self.szridInput.text().strip()
        return all(s for s in [name, ws_key, szrid])
